import numpy as np
import sys
import os
import re
import h5py
import torch

def check_lumo(num_orb, noon):
  lumo = int(round(np.sum(noon)/2))
  if lumo == num_orb:
    print("Fully occupied, no need for MP4")

  return lumo

def calcu_eden_mpn(num_orb, lumo, noe, amp, wfn, A, device):

  wfn = torch.tensor(wfn, device=device).float()
  A = torch.tensor(A, device=device).float()
  amp = torch.tensor(amp, device=device).float()
  e = torch.einsum('ni,na,njb,ijab->n', wfn[:,:lumo], wfn[:,lumo:], A[:,:lumo,lumo:], amp)
  
  return e.numpy()


fwfn = sys.argv[1]
famp = sys.argv[2]
fajb = sys.argv[3]
frg = sys.argv[4]
fsel = sys.argv[5]
fout = sys.argv[6]

# load wave functions from h5py
with h5py.File(fwfn, 'r') as hf:
  typ_orb, num_orb, num_prt, num_nuc = hf['BASIC'][:]
  pos = hf['POS'][:]
  cassign = hf['CASSIGN'][:]
  tassign = hf['TASSIGN'][:]
  exp = hf['EXP'][:]
  noon = hf['NOON'][:]
  noe = hf['NOE'][:]
  nocoef = hf['NOCOEF'][:]

# load 2-ele integration fcidump from npy
amp = np.load(famp)

# load Ajb
Ajb = np.load(fajb)

# load standard grid from npz
rg_grid = np.load(frg)['GRID']
rg_wfn = np.load(frg)['WFN']
rg_wgt = np.load(frg)['WGT']


# load index selection of rg grid
try:
  rg_sel = np.load(fsel)
except:
  try:
    num = int(fsel)
    if num == 0:
      rg_sel = range(len(rg_grid))
    else:
      rg_sel = range(num)
  except:
    start,end = map(int, fsel.split(','))
    rg_sel = range(start,end)


# get the lumo, if fully occupied (lumo is num_orb) return error
lumo = check_lumo(num_orb, noon)

torch.set_num_threads(8)
device = torch.device("cpu")

eden_mp4 = calcu_eden_mpn(num_orb, lumo, noe, amp, rg_wfn[rg_sel], Ajb, device)
np.save(fout, eden_mp4)

print(np.sum(np.multiply(eden_mp4,rg_wgt[rg_sel])))












