import numpy as np
import sys
import os
import re
import h5py
import torch
import time

def calcu_mp3(num_orb, lumo, noe, int2el, device):

  e1 = 0
  ee = torch.zeros((lumo, lumo, num_orb-lumo, num_orb-lumo), device=device)
  
  # orbital energy matrix
  #e = noe[:lumo,None,None,None]+noe[None,:lumo,None,None]-noe[None,None,lumo:num_orb,None]-noe[None,None,None,lumo:num_orb]
  eT = noe[lumo:num_orb,None,None,None]+noe[None,lumo:num_orb,None,None]-noe[None,None,:lumo,None]-noe[None,None,None,:lumo]

  x1 = torch.einsum('rstu,tuab->abrs',int2el[lumo:num_orb,lumo:num_orb,lumo:num_orb,lumo:num_orb],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x2 = torch.einsum('srut,utab->abrs',int2el[lumo:num_orb,lumo:num_orb,lumo:num_orb,lumo:num_orb],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)

  #print(x1==np.swapaxes(x2,2,3))
  y = 2*x1-x2
  ee += (-y)

  
  x1 = torch.einsum('cdab,rscd->abrs',int2el[:lumo,:lumo,:lumo,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x2 = torch.einsum('dcab,srdc->abrs',int2el[:lumo,:lumo,:lumo,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  y = 2*x1-x2
  ee += (-y)

  x1 = torch.einsum('rctb,tsac->abrs',int2el[lumo:num_orb,:lumo,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x2 = torch.einsum('rcta,tscb->abrs',int2el[lumo:num_orb,:lumo,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x3 = torch.einsum('cstb,rtac->abrs',int2el[:lumo,lumo:num_orb,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x4 = torch.einsum('csta,rtcb->abrs',int2el[:lumo,lumo:num_orb,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x5 = torch.einsum('crat,stcb->abrs',int2el[:lumo,lumo:num_orb,:lumo,lumo:num_orb],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x6 = torch.einsum('rctb,tsca->abrs',int2el[lumo:num_orb,:lumo,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x7 = torch.einsum('scat,rtbc->abrs',int2el[lumo:num_orb,:lumo,:lumo,lumo:num_orb],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  x8 = torch.einsum('cstb,rtca->abrs',int2el[:lumo,lumo:num_orb,lumo:num_orb,:lumo],int2el[lumo:num_orb,lumo:num_orb,:lumo,:lumo]/eT)
  y = -4*x1-4*x2+8*x3+2*x4+2*x5+2*x6-4*x7-4*x8
  ee += -(y)
         
  for r in range(lumo,num_orb):
    for s in range(lumo,num_orb):
      for a in range(lumo):
        for b in range(lumo):
          e1 = e1 + ee[a,b,r-lumo,s-lumo]*int2el[a,b,r,s]/(noe[a]+noe[b]-noe[r]-noe[s])

  #amp = ee/(noe[a,None,None,None]+noe[None,b,None,None]-noe[None,None,r,None]-noe[None,None,None,s])
  amp = ee/(noe[:lumo,None,None,None]+noe[None,:lumo,None,None]-noe[None,None,lumo:num_orb,None]-noe[None,None,None,lumo:num_orb])

  return e1.numpy(), amp.numpy()


def main():  
  fwfn = sys.argv[1]
  ffci = sys.argv[2]
    
  # load orbital information
  with h5py.File(fwfn, 'r') as hf:
    typ_orb, num_orb, num_prt, num_nuc = hf['BASIC'][:]
    noon = hf['NOON'][:]
    noe = hf['NOE'][:]

  # load 2-ele terms
  int2el = np.load(ffci)

  # check lumo
  lumo = int(round(np.sum(noon)/2))
  if lumo == num_orb:
    print("Fully occupied, no need for MP3")
  if num_orb > len(int2el):
    print("FCIDUMP only has " + str(len(int2el)) + " orbitals, while WFN gives " + str(num_orb) + " orbitals, use FCIDUMP instead")
    num_orb = len(int2el)

  # calculate MP3 energy
  t1 = time.time()
  torch.set_num_threads(8)
  device = torch.device("cpu")
  int2el =torch.tensor(int2el, device=device).float()
  noe =torch.tensor(noe, device=device).float()

  emp3, amp = calcu_mp3(num_orb, lumo, noe, int2el, device)
  #print(time.time()-t1)
  #print(emp3)
          
  np.save('mp3_amp.npy', amp)
          

if __name__ == "__main__":
  main()
