import numpy as np
import sys
import os
import re
import h5py
import quadpy as qd
from rho_calcu import *

def mkdir(d):
  try:
    os.stat(d)
  except:
    os.mkdir(d)

def trs_lgd(x):
  return (x+1)*5

def calcu_coef(g_lgd, rpt):
  coef = np.power(g_lgd,2)
  coef = np.repeat(coef, rpt)

  return coef

def gen_grid():
  # r
  lgd = qd.line_segment.gauss_legendre(200)
  g_lgd, w_lgd = lgd.points, lgd.weights
  # change from (0,10) to (-1,1) for weight
  g_lgd = trs_lgd(g_lgd)
  w_lgd *= 5 # dx = 5dx'

  # theta & phi
  lbd = qd.sphere.lebedev_031()
  g_lbd, w_lbd = lbd.points, lbd.weights
  
  grid = np.outer(g_lgd, g_lbd).reshape(-1,3)
  wgt = np.outer(w_lgd, w_lbd).reshape(-1)
  coef = calcu_coef(g_lgd, len(g_lbd))

  return g_lgd, grid, coef * wgt

def calcu_nu(muAB, aAB, r): # recursion r times
  if r == 0:
    return muAB + aAB*(1-np.power(muAB,2))
  else:
    nuAB = calcu_nu(muAB, aAB, r-1)
    return nuAB*(1.5-0.5*np.power(nuAB,2))
  
def Becke_scheme(grid, pos, num_nuc, chi):
  iwgt = np.empty((num_nuc, len(grid)))
  for gA in range(num_nuc):
    grid_A = np.add(grid, pos[gA])
    nuAB = - np.ones((num_nuc, num_nuc, len(grid))) # initiate with -1 so 0.5*(1-nuAB)=1 does not change the product

    for A in range(num_nuc):
      rA = np.linalg.norm(np.subtract(grid_A, pos[A]), axis=-1)

      for B in range(num_nuc):
        if B is A:
          continue
        rB = np.linalg.norm(np.subtract(grid_A, pos[B]), axis=-1)
        rAB = np.linalg.norm(pos[A] - pos[B])
        muAB = np.subtract(rA, rB) / rAB
        chiAB = chi[A] / chi[B]
        aAB = 0.25*(1-np.power(chiAB,2)) / chiAB
        nuAB[A,B] = calcu_nu(muAB, aAB, r)

    # now calculate pA, we only interest in weight of gA (grid on A) on nuclear A
    pA = np.prod(0.5*(1-nuAB), axis=1)
    iwgt[gA] = np.divide(pA[gA], np.sum(pA, axis=0))

  return iwgt
      
    

fwfn = sys.argv[1]
fchi = sys.argv[2]
r = int(sys.argv[3])

# load wave functions from h5py
with h5py.File(fwfn, 'r') as hf:
  _, _, _, num_nuc = hf['BASIC'][:]
  pos = hf['POS'][:]

try:
  num_nuc = int(sys.argv[4])
except:
  print('use all atoms as integration centers')

# load chi number for each atom
chi = np.loadtxt(fchi)

# construct grid from Legendre quadrature Lebedev sphere quadrature on (0,0,0)
_, grid, coef_wgt = gen_grid()


# shift the grid to every nuclear position
# save the grid
Becke_grid = []
for n in range(num_nuc):
  Becke_grid.extend(np.add(grid, pos[n]))
Becke_grid = np.array(Becke_grid)  
np.save('Becke_grid.npy', Becke_grid)

# generate integration weight according to the grid position to each nuclear (Becke's scheme)
# iwgt: num_nuc * len(grid)
iwgt = Becke_scheme(grid, pos, num_nuc, chi)

# save the overall weight
Becke_wgt = []
for n in range(num_nuc):
  Becke_wgt.extend(np.multiply(coef_wgt, iwgt[n]))
Becke_wgt = 4 * np.pi * np.array(Becke_wgt)
np.save('Becke_wgt.npy', Becke_wgt)

#print(Becke_grid.shape)
#print(Becke_wgt.shape)

print(str(len(grid)) + ' * ' + str(num_nuc) + ' Becke grids are generated!')



